from dorothy_sdk.session import Session
from requests import Session


class Image:
    resource = "images"

    def __init__(self, dataset_name: str, image_url: str, project_id: str,
                 insertion_date: str, metadata: dict, date_acquisition: str, number_reports: int,
                 session: Session, host: str, dataset_id: str = None, *args, **kwargs):
        self.dataset_name = dataset_name
        self.image_url = image_url
        self.project_id = project_id
        self.insertion_date = insertion_date
        self.metadata = metadata
        self.date_acquisition = date_acquisition
        self.number_reports = number_reports
        self.dataset_id = dataset_id
        self._session: Session = session
        self._service_host = host
        self.content = None
        self.image_format = None
        self.download_parameters = None

    def download_image(self, width: int = None, height: int = None, gray_scale: bool = False) -> bytes:
        parameters = {}
        if width:
            parameters["width"] = width
        if height:
            parameters["height"] = height
        parameters["grayscale"] = gray_scale
        self.download_parameters = parameters
        request = self._session.get(self.image_url, params=parameters)
        request.raise_for_status()
        self.image_format = request.headers.get("Content-Type", "image/png").split("/")[-1]
        if request.status_code == 200:
            self.content = request.content
            return request.content
        else:
            raise RuntimeError("Could not download image")

    def save(self, image_path: str = None):
        if not self.content:
            self.download_image()
        if image_path:
            with open(image_path, mode='w') as target_file:
                target_file.write(self.content)
            return
        else:
            with open(f"{self.dataset_name}_{self.project_id}.{self.image_format}", mode="w") as target_file:
                target_file.write(self.content)
            return




import os
from requests import Session


class Session(Session):
    _token = None
    _token_file = '/tmp/dorotysdk/credentials.txt'
    _environment_variable_name = 'DOROTYSDK_ACCESS_TOKEN'

    def __init__(self, token: str = None, **options):
        super().__init__()
        if not token:
            self._fetch_token(**options)
        else:
            self._token = token
        self.headers.update({"Authorization": f"Token {self._token}"})

    def set_token(self, token: str) -> None:
        """
        Metodo responsável por fazer o set da variavel _token
        :param token: String com o token de acesso ao Dorothy
        :return: None
        """
        self._token = token

    def _fetch_token(self, **options):
        if options.get('path', ''):
            if os.path.exists(options.get('path')):
                self._token_file = options.get('path')
            else:
                FileNotFoundError("Credentials file passed as parameter not found.")

        token = os.environ.get(self._environment_variable_name, '')
        if not token:
            if os.path.exists(self._token_file):
                with open(self._token_file, mode='r', encoding='utf-8') as credentials_file:
                    self._token = credentials_file.read().replace('\n', '').replace('\t', '').replace('\r', '').strip()
            else:
                raise RuntimeError("Access credential not found. The credential can be entered either as a parameter "
                                   "or via a credentials file or environment variable")
        else:
            self._token = token



import warnings
from typing import List

from dorothy_sdk.resources import Dataset, Image
from dorothy_sdk.session import Session
from dorothy_sdk.utils import url_join


class Client:
    _service_host: str = "https://dorothy-image.lps.ufrj.br"

    def __init__(self, token=None, **kwargs):
        self.session = Session(token, **kwargs)
        if kwargs.get("host"):
            warnings.warn(f"Modifying host from {self._service_host} to {kwargs.get('host')}")
            self._service_host = kwargs.get("host")

    def get_datasets(self) -> List[Dataset]:
        request = self.session.get(url_join(self._service_host, Dataset.resource))
        request.raise_for_status()
        datasets = []
        for element in request.json():
            datasets.append(Dataset(session=self.session, host=self._service_host, **element))
        return datasets

    def dataset(self, dataset_id: str) -> Dataset:
        request = self.session.get(url_join(self._service_host, Dataset.resource), params={"search": dataset_id})
        request.raise_for_status()
        if request.status_code == 200:
            return Dataset(session=self.session, host=self._service_host, **[dataset for dataset in request.json() if dataset['name'] == dataset_id][0])
        elif request.status_code == 404:
            return None
        else:
            raise RuntimeError("Unable to fetch dataset")

    def image(self, image_id: str) -> Image:
        request = self.session.get(url_join(self._service_host, Image.resource), params={"search": image_id})
        request.raise_for_status()
        if request.status_code == 200:
            return Image(session=self.session, host=self._service_host, **request.json()[0])
        elif request.status_code == 404:
            return None
        else:
            raise RuntimeError("Unable to fetch image")
